#ifndef NBODYSIM_HPP
#define NBODYSIM_HPP

#include "tree.hpp"
#include <string>

class nBodySim {
public:
    // Constructor reads in data from datafile
    nBodySim(std::string datafile);
    // Destructor deletes arrays 
    ~nBodySim();

    // Run simulation for nSteps time steps of size dt, integration with leapfrog algorithm
    void runSimulation(double dt, unsigned nSteps);

    // Loops over all pairs of particles and calculates forces 
    void updateForces();
    // Use treecode and multipole expansion to calculate forces
    void treeUpdateForces();
    // Loops over all pairs of particles and calculates forces, checks if forces are correct
    double verifyForces() const;

    // Leapfrog: update velocities
    void kick(double dt);
    // Leapfrog: update positions
    void drift(double dt);
    // Update tree
    void updateTree();

    // Loop over all pairs of particles and calculate mean interparticle distance
    double calculateMeanInterparticleDistance() const;
    // Loop over all force vectors and calculate mean force magnitude
    double calculateMeanForceMagnitude() const;
    // loop over every particle and calculate the mean distance from the center
    double calculateMeanCenterDistance() const;
    // Loop over all particles and calculate the total energy
    double calculateTotalEnergy() const;

    // Writes data to file, every row is a particle
    void write2file(const double* array, std::ofstream& file, unsigned dim, unsigned skip) const;
    // Write current state of simulation to file
    void saveState2file(unsigned step, std::ofstream& file) const;

    // Functions to get private variables
    double getMass() { return mass_; }
    double* getPositions() { return positions_; }
    double* getForces() { return forces_; }
    unsigned getNParticles() const { return nParticles_; }

private:
    Tree* tree_;

    unsigned nParticles_;
    // 3d vectors are stored as a array of length 3*nParticles_ in the format x1, y1, z1, x2, y2, z2, ...
    double mass_;
    double* positions_;
    double* velocities_;
    double softening_;
    double* potential_;
    double* forces_;
};

#endif /* NBODYSIM_HPP */